package com.synnex.notification.activity;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.ComponentName;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.ServiceConnection;
import android.content.SharedPreferences.Editor;
import android.os.Bundle;
import android.os.IBinder;
import android.preference.PreferenceManager;
import android.util.Log;
import android.view.KeyEvent;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.EditText;
import android.widget.Toast;

import com.synnex.notification.R;
import com.synnex.notification.service.NotificationBinder;
import com.synnex.notification.service.NotificationService;
import com.synnex.notification.util.Config;
import com.synnex.notification.util.StringUtil;

public class MainActivity extends Activity implements OnClickListener
{
	private final String tag = "MainActivity";
	private final int DIALOG_EXIT_APP = 1;
	
	private View startBtn;
	private View stopBtn;
	private EditText usernameView;
	
	public void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);
		setContentView(R.layout.main);
		
		startBtn = findViewById(R.id.btn_start);
		stopBtn = findViewById(R.id.btn_stop);
		startBtn.setOnClickListener(this);
		stopBtn.setOnClickListener(this);
		
		usernameView = (EditText)findViewById(R.id.username);
		String username = PreferenceManager.getDefaultSharedPreferences(this).getString(Config.KEY_USER_NAME, null);
		if(!StringUtil.isEmpty(username))
		{
			usernameView.setText(username);
		}
		
		startNotificationService();
		changeButtonStatus();
	}
	public void onClick(View v)
	{
		if(v == startBtn)
		{
			String username = usernameView.getText().toString();
			if(StringUtil.isEmpty(username))
			{
				usernameView.setError("Please input your username!");
				return;
			}
			Editor editor = PreferenceManager.getDefaultSharedPreferences(this).edit();
			editor.putString(Config.KEY_USER_NAME, username);
			editor.commit();
			
			if(notificationService != null)
			{
				notificationService.startListen();
				changeButtonStatus();
			}
			else
			{
				Toast.makeText(this, "## Bind service failure", Toast.LENGTH_SHORT).show();
			}
		}
		else if(v == stopBtn)
		{
			if(notificationService != null)
			{
				notificationService.stopListen();
				changeButtonStatus();
			}
			else
			{
				Toast.makeText(this, "## Bind service failure", Toast.LENGTH_SHORT).show();
			}
		}
	}
	private void changeButtonStatus()
	{
		if(notificationService == null)
		{
			startBtn.setEnabled(true);
			stopBtn.setEnabled(false);
		}
		else
		{
			if(notificationService.isListening())
			{
				startBtn.setEnabled(false);
				stopBtn.setEnabled(true);
			}
			else
			{
				startBtn.setEnabled(true);
				stopBtn.setEnabled(false);
			}
		}
	}
	private void startNotificationService()
	{
		Intent intent = new Intent(this, NotificationService.class);
		intent.setAction(NotificationService.class.getName());
		intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
		startService(intent);
		
		bindService(intent, conn, BIND_AUTO_CREATE);
	}
	private NotificationService notificationService;
	
	private ServiceConnection conn = new ServiceConnection()
	{
		@Override
		public void onServiceConnected(ComponentName name, IBinder binder)
		{
			Log.i(tag, "## NotificationService connected ##");
			if(binder != null && binder instanceof NotificationBinder)
			{
				notificationService = ((NotificationBinder)binder).getService();
				changeButtonStatus();
			}
		}
		@Override
		public void onServiceDisconnected(ComponentName name)
		{
			Log.i(tag, "## NotificationService disconnected ##");
		}
	};
	public boolean onKeyDown(int keyCode, KeyEvent event)
	{
		if (keyCode == KeyEvent.KEYCODE_BACK && event.getRepeatCount() == 0)
		{
			showDialog(DIALOG_EXIT_APP);
			return true;
		}
		return super.onKeyDown(keyCode, event);
	}
	protected Dialog onCreateDialog(int id)
	{
		switch (id)
		{
			case DIALOG_EXIT_APP:
				return new AlertDialog.Builder(this)
					.setTitle("Exit")
					.setMessage("Do you want to exit?")
					.setPositiveButton("Exit",
						new DialogInterface.OnClickListener()
						{
							public void onClick(DialogInterface dialog, int whichButton)
							{
								android.os.Process.killProcess(android.os.Process.myPid());
							}
						})
					.setNegativeButton("Cancel", null).create();
			default:
				break;
		}
		return super.onCreateDialog(id);
	}
}