package com.synnex.notification.service;

import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.preference.PreferenceManager;
import android.util.Log;

import com.synnex.notification.R;
import com.synnex.notification.http.HTTPNotification;
import com.synnex.notification.http.OnNotificationListener;
import com.synnex.notification.util.Config;
import com.synnex.notification.util.StringUtil;

public class NotificationService extends Service implements OnNotificationListener
{
	private final String tag = "NotificationService";
	
	private final int WHAT_NOTIFICATION_SUCCESS = 1;
	private final int WHAT_NOTIFICATION_FAILURE = 2;
	
	private IBinder binder = new NotificationBinder(this);
	
	public IBinder onBind(Intent intent)
	{
		Log.i(tag, "## NotificationService is onBind ##");
		return binder;
	}
	
	public void onCreate()
	{
		Log.i(tag, "## NotificationService is onCreate ##");
		startListen();
		super.onCreate();
	}
	
	public void onStart(Intent intent, int startId)
	{
		Log.i(tag, "## NotificationService is onStart ##");
		super.onStart(intent, startId);
	}
	
	private Handler handler = new Handler()
	{
		public void handleMessage(Message msg)
		{
			String title = "";
			switch (msg.what)
			{
				case WHAT_NOTIFICATION_SUCCESS:
					title = "New Message";
					break;
				case WHAT_NOTIFICATION_FAILURE:
					title = "Error";
					break;
				default:
					break;
			}
			if(!StringUtil.isEmpty(title))
			{
				NotificationManager notificationManager = (NotificationManager)getSystemService(NOTIFICATION_SERVICE);
				Notification notification = new Notification(R.drawable.ic_launcher, title, System.currentTimeMillis());
				notification.defaults = Notification.DEFAULT_SOUND;
				notification.flags = Notification.FLAG_AUTO_CANCEL;
				PendingIntent intent = PendingIntent.getActivity(getApplicationContext(), 0, new Intent(), 0);
				notification.setLatestEventInfo(getApplicationContext(), title, msg.obj + "", intent);
				notificationManager.notify(0, notification);
			}
		};
	};
	private HTTPNotification httpNotification;
	private boolean isListening;
	public void startListen()
	{
		stopListen();
		final String username = getUsername();
		if(StringUtil.isEmpty(username))
		{
			Log.i(tag, "## user name is empty ##");
			return;
		}
		
		httpNotification = new HTTPNotification("http://192.168.85.178:8080/notification/notification.do", username);
		httpNotification.setOnNotificationListener(this);
		httpNotification.start();
		isListening = true;
	}
	
	public void stopListen()
	{
		try
		{
			if(isListening())
			{
				httpNotification.stop();
				httpNotification = null;
			}
			Log.v(tag, "## stop listen success!");
		}
		catch (Exception e)
		{
			Log.e(tag, "stop listen error: ", e);
		}
		isListening = false;
	}
	
	private String getUsername()
	{
		SharedPreferences preferences = PreferenceManager.getDefaultSharedPreferences(getApplicationContext());
		return preferences.getString(Config.KEY_USER_NAME, null);
	}
	public void onError(String msg)
	{
		Message message = handler.obtainMessage();
		message.obj = msg;
		message.what = WHAT_NOTIFICATION_FAILURE;
		handler.sendMessage(message);
	}
	public void onNotification(String msg)
	{
		Message message = handler.obtainMessage();
		message.obj = msg;
		message.what = WHAT_NOTIFICATION_SUCCESS;
		handler.sendMessage(message);
	}
	public boolean isListening()
	{
		return isListening;
	}
}